@@ -2,11 +2,11 @@
 NULL
 
 
-#' @title abstract class to represent a clustering result
-#' 
+#' @title Abstract class to represent a clustering result
+#'
 #' @description
 #' An S4 abstract class to represent an icl fit of a clustering model.
-#' 
+#'
 #' @slot K a numeric vector of length 1 which correspond to the number of clusters
 #' @slot icl a numeric vector of length 1 which store the the icl value
 #' @slot cl a numeric vector of length N which store the clusters labels
@@ -15,25 +15,20 @@ NULL
 #' @slot move_mat binary matrix which store move constraints
 #' @slot train_hist a data.frame to store training history (format depends on the used algorithm used).
 #' @slot name generative model name
-#' @seealso \code{\link{sbm_fit-class}}, \code{\link{dcsbm_fit-class}}, \code{\link{co_dcsbm_fit-class}}
-#' @export 
-setClass("icl_fit",slots = list(name="character",K="numeric",obs_stats="list",obs_stats_cst="list",icl="numeric",cl="numeric",train_hist="data.frame",move_mat = "dgCMatrix"))
+#' @export
+setClass("IclFit", slots = list(name = "character", K = "numeric", obs_stats = "list", obs_stats_cst = "list", icl = "numeric", cl = "numeric", train_hist = "data.frame", move_mat = "dgCMatrix"))
 
 
 
 
-#' @title  abstract class to represent a hierarchical clustering result
-#' 
+#' @title  Abstract class to represent a hierarchical clustering result
+#'
 #' @description
 #' An S4 class to represent a hierarchical path of solution.
-#' 
+#'
 #' @slot path a list of merge moves describing the hierarchy of merge followed to complete totally the merge path.
 #' @slot tree a tree representation of the merges.
 #' @slot ggtree a data.frame for easy plotting of the dendrogram
 #' @slot logalpha a numeric value which corresponds to the starting value of log(alpha).
-#' @seealso \code{\link{sbm_path-class}}, \code{\link{dcsbm_path-class}}, \code{\link{co_dcsbm_path-class}}
 #' @export
-setClass("icl_path",slots=list(path="list",tree="numeric",ggtree="data.frame",logalpha="numeric"))
-
-
-
+setClass("IclPath", slots = list(path = "list", tree = "numeric", ggtree = "data.frame", logalpha = "numeric"))
