@@ -1,65 +1,72 @@
 // [[Rcpp::depends(RcppArmadillo)]]
 #include "gicl_tools.h"
 #include "MergeMat.h"
-#include "IclModel.h"
+#include "IclModelEmission.h"
 #include "DcSbm.h"
 using namespace Rcpp;
 
-
-
-DcSbm::DcSbm(const arma::sp_mat& xp,S4 modeli,arma::vec& clt,bool verb){
-  model = modeli;
-  alpha = modeli.slot("alpha");
+DcSbm::DcSbm(const arma::sp_mat  & xp,S4 modeli,bool verb){
+  if(Rcpp::traits::is_nan<REALSXP>(modeli.slot("p"))){
+    model=clone(modeli);
+  }else{
+    model = modeli;
+  }
+  
   x  = xp;
   xt = xp.t();
   N  = x.n_rows;
-  set_cl(clt);
   if(Rcpp::traits::is_nan<REALSXP>(model.slot("p"))){
-    p= arma::accu(x_counts)/(N*N);
-    model.slot("p") = p;
+    p= arma::accu(x)/(N*N);
+    model.slot("p")=p;
   }else{
     p = model.slot("p");
   }
   verbose=verb;
-  // cst to add 
-  cst = 0;
+
 }
 
-void DcSbm::set_cl(arma::vec clt){
-  cl = clt;
-  K = arma::max(cl)+1;
-  x_counts = gsum_mat(cl,x,K);
-  counts = count(cl,K);
+
+
+void DcSbm::set_cl(arma::uvec clt){
+  K = arma::max(clt)+1;
+  x_counts = gsum_mat(clt,x,K);
+  counts = count(clt,K);
   din = sum(x_counts).t();
   dout = sum(x_counts.t()).t();
+  double cst_denom = 0;
+  int nlinks =0;
+  for (arma::sp_mat::const_iterator i = x.begin(); i != x.end(); ++i) {
+    cst_denom+=lgamma(*i+1);
+    nlinks += *i;
+  }
+  cst = arma::accu(lgamma(sum(x)+1))+arma::accu(lgamma(sum(xt)+1))-cst_denom + nlinks*log(p);
 }
 
 double DcSbm::icl_emiss(const List & obs_stats){
-
-  arma::vec counts =as<arma::vec>(obs_stats["counts"]);
+  
+  arma::vec counts = as<arma::vec>(obs_stats["counts"]);
   arma::vec din =as<arma::vec>(obs_stats["din"]);
   arma::vec dout =as<arma::vec>(obs_stats["dout"]);
   arma::mat edges_counts =as<arma::mat>(obs_stats["x_counts"]);
   arma::mat matcount = counts*counts.t();
   // lets go
-
-
-  double icl_emiss = accu(lgamma(counts)-lgamma(counts+din)+din % log(counts))+accu(lgamma(counts)-lgamma(counts+dout)+dout % log(counts));
-
+  
+  
+  double icl_emiss = arma::accu(lgamma(counts)-lgamma(counts+din)+din % log(counts))+accu(lgamma(counts)-lgamma(counts+dout)+dout % log(counts));
   icl_emiss=icl_emiss + arma::accu(lgamma(edges_counts+1)-(edges_counts+1) % log(p*matcount+1));
-
   return icl_emiss+cst;
 }
 
-double DcSbm::icl_emiss(const List & obs_stats,int oldcl,int newcl){
+double DcSbm::icl_emiss(const List & obs_stats,int oldcl,int newcl, bool dead_cluster){
+
   arma::vec counts =as<arma::vec>(obs_stats["counts"]);
   arma::vec din =as<arma::vec>(obs_stats["din"]);
   arma::vec dout =as<arma::vec>(obs_stats["dout"]);
   arma::mat edges_counts =as<arma::mat>(obs_stats["x_counts"]);
-  arma::mat si = submatcross(oldcl,newcl,counts.n_rows);
+  arma::umat si = submatcross(oldcl,newcl,K);
   double icl_emiss = lgamma(counts(newcl))-lgamma(counts(newcl)+din(newcl))+din(newcl)*log(counts(newcl));
   icl_emiss += lgamma(counts(newcl))-lgamma(counts(newcl)+dout(newcl))+dout(newcl)*log(counts(newcl));
-  if(counts(oldcl)!=0){
+  if(!dead_cluster){
     icl_emiss += lgamma(counts(oldcl))-lgamma(counts(oldcl)+dout(oldcl))+dout(oldcl)*log(counts(oldcl));
     icl_emiss += lgamma(counts(oldcl))-lgamma(counts(oldcl)+din(oldcl))+din(oldcl)*log(counts(oldcl));
   }
@@ -70,12 +77,13 @@ double DcSbm::icl_emiss(const List & obs_stats,int oldcl,int newcl){
     k=si(i,0);
     l=si(i,1);
     if(counts(k)*counts(l)!=0){
-        cc = counts(k)*counts(l);
+      cc = counts(k)*counts(l);
       // lets go
-        icl_emiss += lgamma(edges_counts(k,l)+1)-(edges_counts(k,l)+1)*log(p*cc+1);
+      icl_emiss += lgamma(edges_counts(k,l)+1)-(edges_counts(k,l)+1)*log(p*cc+1);
     }
     
   }
+  
   return icl_emiss+cst;
 }
 
@@ -88,12 +96,12 @@ List DcSbm::get_obs_stats(){
 
 List DcSbm::get_obs_stats_cst(){
   arma::sp_mat din_node = sum(x).t();
-  arma::sp_mat dout_node = sum(x.t()).t();
+  arma::sp_mat dout_node = sum(xt).t();
   return List::create(Named("din_node", din_node),Named("dout_node", dout_node));
 }
 
 
-arma::mat DcSbm::delta_swap(int i,arma::uvec iclust){
+arma::vec DcSbm::delta_swap(int i,arma::uvec & cl, bool almost_dead_cluster, arma::uvec iclust, int K){
   int self=x(i,i);
   int oldcl = cl(i);
   arma::sp_mat col_sum = gsum_col(cl,x,i,K);
@@ -126,8 +134,8 @@ arma::mat DcSbm::delta_swap(int i,arma::uvec iclust){
       new_dout(oldcl) = new_dout(oldcl)-cdout;
       new_dout(k) = new_dout(k)+cdout;
       List new_stats = List::create(Named("counts", new_counts),Named("din", new_din),Named("dout", new_dout), Named("x_counts", new_ec));
-      delta(k)=icl(new_stats,oldcl,k)-icl(old_stats,oldcl,k);
-
+      delta(k)=icl_emiss(new_stats,oldcl,k,almost_dead_cluster)-icl_emiss(old_stats,oldcl,k,false);
+      
     }
     
   }
@@ -135,45 +143,35 @@ arma::mat DcSbm::delta_swap(int i,arma::uvec iclust){
 }
 
 
-void DcSbm::swap_update(const int i,const int newcl){
+void DcSbm::swap_update(const int i,arma::uvec &  cl,bool dead_cluster,const int newcl){
   int self=x(i,i);
   int oldcl = cl(i);
   arma::sp_mat col_sum = gsum_col(cl,x,i,K);
   col_sum(oldcl)=col_sum(oldcl)-self;
   arma::sp_mat row_sum = gsum_col(cl,xt,i,K);
   row_sum(oldcl)=row_sum(oldcl)-self;
-  arma::mat new_ec = x_counts;
   int cdin = arma::accu(col_sum)+self;
   int cdout = arma::accu(row_sum)+self;
-  new_ec.col(newcl) = new_ec.col(newcl)+col_sum;
-  new_ec.row(newcl) = new_ec.row(newcl)+row_sum.t();
-  new_ec.col(oldcl) = new_ec.col(oldcl)-col_sum;
-  new_ec.row(oldcl) = new_ec.row(oldcl)-row_sum.t();
-  new_ec(newcl,newcl)=new_ec(newcl,newcl)+self;
-  new_ec(oldcl,oldcl)=new_ec(oldcl,oldcl)-self;
-  arma::mat new_counts = update_count(counts,oldcl,newcl);
-  arma::vec new_din = din;
-  new_din(oldcl) = new_din(oldcl)-cdin;
-  new_din(newcl) = new_din(newcl)+cdin;
-  arma::vec new_dout = dout;
-  new_dout(oldcl) = new_dout(oldcl)-cdout;
-  new_dout(newcl) = new_dout(newcl)+cdout;
-  cl(i)=newcl;
-  if(new_counts(oldcl)==0){
-    counts = new_counts.elem(arma::find(arma::linspace(0,K-1,K)!=oldcl));
-    din = new_din.elem(arma::find(arma::linspace(0,K-1,K)!=oldcl));
-    dout = new_dout.elem(arma::find(arma::linspace(0,K-1,K)!=oldcl));
-    x_counts = new_ec(arma::find(arma::linspace(0,K-1,K)!=oldcl),arma::find(arma::linspace(0,K-1,K)!=oldcl));
-    cl.elem(arma::find(cl>oldcl))=cl.elem(arma::find(cl>oldcl))-1;
+  x_counts.col(newcl) = x_counts.col(newcl)+col_sum;
+  x_counts.row(newcl) = x_counts.row(newcl)+row_sum.t();
+  x_counts.col(oldcl) = x_counts.col(oldcl)-col_sum;
+  x_counts.row(oldcl) = x_counts.row(oldcl)-row_sum.t();
+  x_counts(newcl,newcl)=x_counts(newcl,newcl)+self;
+  x_counts(oldcl,oldcl)=x_counts(oldcl,oldcl)-self;
+  counts = update_count(counts,oldcl,newcl);
+  din(oldcl) = din(oldcl)-cdin;
+  din(newcl) = din(newcl)+cdin;
+  dout(oldcl) = dout(oldcl)-cdout;
+  dout(newcl) = dout(newcl)+cdout;
+  if(dead_cluster){
+    counts.shed_row(oldcl);
+    din.shed_row(oldcl);
+    dout.shed_row(oldcl);
+    x_counts = x_counts(arma::find(arma::linspace(0,K-1,K)!=oldcl),arma::find(arma::linspace(0,K-1,K)!=oldcl));
     --K;
-  }else{
-    counts=new_counts;
-    x_counts=new_ec;
-    din=new_din;
-    dout=new_dout;
   }
-
-
+  
+  
 }
 
 
@@ -182,7 +180,7 @@ double DcSbm::delta_merge(int k, int l){
   List old_stats = List::create(Named("counts", counts), Named("din", din),Named("dout", dout),Named("x_counts", x_counts));
   
   arma::mat new_ec = x_counts;
-  arma::mat new_counts = counts;
+  arma::vec new_counts = counts;
   new_counts(l)=new_counts(k)+new_counts(l);
   new_counts(k)=0;
   new_ec.col(l) = new_ec.col(k)+new_ec.col(l);
@@ -192,8 +190,7 @@ double DcSbm::delta_merge(int k, int l){
   arma::vec new_dout = dout;
   new_dout(l) = new_dout(k)+new_dout(l);
   List new_stats = List::create(Named("counts", new_counts),Named("din", new_din),Named("dout", new_dout), Named("x_counts", new_ec));
-  double delta=icl(new_stats,k,l)-icl(old_stats,k,l);
-  
+  double delta=icl_emiss(new_stats,k,l,true)-icl_emiss(old_stats,k,l,false);
   return delta;
 }
 
@@ -270,9 +267,7 @@ double DcSbm::delta_merge_correction(int k,int l,int obk,int obl,const List & ol
       
     }
   }
-  if(l==obk){
-    icl_cor=0;
-  }
+
   return icl_cor;
   
   
@@ -281,20 +276,21 @@ double DcSbm::delta_merge_correction(int k,int l,int obk,int obl,const List & ol
 
 
 void DcSbm::merge_update(int k,int l){
-  cl(arma::find(cl==k))=arma::ones(counts(k),1)*l;
-  cl.elem(arma::find(cl>k))=cl.elem(arma::find(cl>k))-1;
   counts(l) = counts(k)+counts(l);
-  counts    = counts.elem(arma::find(arma::linspace(0,K-1,K)!=k));
-
+  counts.shed_row(k);
+  
   x_counts.col(l) = x_counts.col(k)+x_counts.col(l);
   x_counts.row(l) = x_counts.row(k)+x_counts.row(l);
   x_counts = x_counts(arma::find(arma::linspace(0,K-1,K)!=k),arma::find(arma::linspace(0,K-1,K)!=k));
   
   din(l) = din(l)+din(k);
-  din = din.elem(arma::find(arma::linspace(0,K-1,K)!=k));
+  din.shed_row(k);
   dout(l) = dout(l)+dout(k);
-  dout = dout.elem(arma::find(arma::linspace(0,K-1,K)!=k));
+  dout.shed_row(k);
   
   
   --K;
 }
+
+
+
