@@ -1,32 +1,30 @@
 // [[Rcpp::depends(RcppArmadillo)]]
 #include "gicl_tools.h"
 #include "MergeMat.h"
-#include "IclModel.h"
+#include "IclModelEmission.h"
 #include "Sbm.h"
 using namespace Rcpp;
 
-
-
-Sbm::Sbm(arma::sp_mat& xp,S4 modeli,arma::vec& clt,bool verb){
-  model= modeli;
-  alpha = model.slot("alpha");
+Sbm::Sbm(const arma::sp_mat  & xp,S4 modeli,bool verb){
+  model = modeli;
+  x  = xp;
+  xt = xp.t();
+  N  = x.n_rows;
   a0 = model.slot("a0");
   b0 = model.slot("b0");
   x  = xp;
   xt = xp.t();
-  N  = x.n_rows;
-  set_cl(clt);
   verbose=verb;
 }
 
-void Sbm::set_cl(arma::vec clt){
-  cl = clt;
-  K = arma::max(cl)+1;
-  x_counts = gsum_mat(cl,x,K);
-  counts = count(cl,K);
-}
 
 
+void Sbm::set_cl(arma::uvec clt){
+  K = arma::max(clt)+1;
+  x_counts = gsum_mat(clt,x,K);
+  counts = count(clt,K);
+}
+
 double Sbm::icl_emiss(const List & obs_stats){
   arma::vec counts =as<arma::vec>(obs_stats["counts"]);
   arma::mat edges_counts =as<arma::mat>(obs_stats["x_counts"]);
@@ -37,10 +35,10 @@ double Sbm::icl_emiss(const List & obs_stats){
   return icl_emiss;
 }
 
-double Sbm::icl_emiss(const List & obs_stats,int oldcl,int newcl){
+double Sbm::icl_emiss(const List & obs_stats,int oldcl,int newcl, bool dead_cluster){
   arma::vec counts =as<arma::vec>(obs_stats["counts"]);
   arma::mat edges_counts =as<arma::mat>(obs_stats["x_counts"]);
-  arma::mat si = submatcross(oldcl,newcl,counts.n_rows);
+  arma::umat si = submatcross(oldcl,newcl,counts.n_rows);
   double icl_emiss = 0;
   int k = 0;
   int l = 0;
@@ -49,20 +47,25 @@ double Sbm::icl_emiss(const List & obs_stats,int oldcl,int newcl){
     k=si(i,0);
     l=si(i,1);
     if(counts(k)*counts(l)!=0){
-        cc = counts(k)*counts(l);
-        icl_emiss += lgamma(a0+edges_counts(k,l))+lgamma(b0+cc-edges_counts(k,l))+lgamma(a0+b0)-lgamma(a0)-lgamma(b0)-lgamma(a0+b0+cc);  
+      cc = counts(k)*counts(l);
+      icl_emiss += lgamma(a0+edges_counts(k,l))+lgamma(b0+cc-edges_counts(k,l))+lgamma(a0+b0)-lgamma(a0)-lgamma(b0)-lgamma(a0+b0+cc);  
     }
     
   }
   return icl_emiss;
+  
+  
 }
 
 
+
 List Sbm::get_obs_stats(){
   return List::create(Named("counts", counts), Named("x_counts", x_counts));
 }
 
-arma::mat Sbm::delta_swap(int i,arma::uvec iclust){
+
+
+arma::vec Sbm::delta_swap(int i,arma::uvec & cl, bool almost_dead_cluster, arma::uvec iclust, int K){
   int self=x(i,i);
   int oldcl = cl(i);
   arma::sp_mat col_sum = gsum_col(cl,x,i,K);
@@ -87,39 +90,36 @@ arma::mat Sbm::delta_swap(int i,arma::uvec iclust){
       new_ec(oldcl,oldcl)=new_ec(oldcl,oldcl)-self;
       arma::vec new_counts = update_count(counts,oldcl,k);
       List new_stats = List::create(Named("counts", new_counts), Named("x_counts", new_ec));
-      delta(k)=icl(new_stats,oldcl,k)-icl(old_stats,oldcl,k);
+      delta(k)=icl_emiss(new_stats,oldcl,k,almost_dead_cluster)-icl_emiss(old_stats,oldcl,k,false);
     }
   }
   return delta;
+  
 }
 
 
-void Sbm::swap_update(const int i,const int newcl){
+void Sbm::swap_update(const int i,arma::uvec &  cl,bool dead_cluster,const int newcl){
   int self=x(i,i);
   int oldcl = cl(i);
   arma::sp_mat col_sum = gsum_col(cl,x,i,K);
   col_sum(oldcl)=col_sum(oldcl)-self;
   arma::sp_mat row_sum = gsum_col(cl,xt,i,K);
   row_sum(oldcl)=row_sum(oldcl)-self;
-  arma::mat new_ec = x_counts;
-  new_ec.col(newcl) = new_ec.col(newcl)+col_sum;
-  new_ec.row(newcl) = new_ec.row(newcl)+row_sum.t();
-  new_ec.col(oldcl) = new_ec.col(oldcl)-col_sum;
-  new_ec.row(oldcl) = new_ec.row(oldcl)-row_sum.t();
-  new_ec(newcl,newcl)=new_ec(newcl,newcl)+self;
-  new_ec(oldcl,oldcl)=new_ec(oldcl,oldcl)-self;
-  arma::mat new_counts = update_count(counts,oldcl,newcl);
-  cl(i)=newcl;
-  if(new_counts(oldcl)==0){
-    counts = new_counts.elem(arma::find(arma::linspace(0,K-1,K)!=oldcl));
-    x_counts = new_ec(arma::find(arma::linspace(0,K-1,K)!=oldcl),arma::find(arma::linspace(0,K-1,K)!=oldcl));
-    cl.elem(arma::find(cl>oldcl))=cl.elem(arma::find(cl>oldcl))-1;
+  
+  x_counts.col(newcl) = x_counts.col(newcl)+col_sum;
+  x_counts.row(newcl) = x_counts.row(newcl)+row_sum.t();
+  x_counts.col(oldcl) = x_counts.col(oldcl)-col_sum;
+  x_counts.row(oldcl) = x_counts.row(oldcl)-row_sum.t();
+  x_counts(newcl,newcl)=x_counts(newcl,newcl)+self;
+  x_counts(oldcl,oldcl)=x_counts(oldcl,oldcl)-self;
+  counts = update_count(counts,oldcl,newcl);
+  if(dead_cluster){
+    counts.shed_row(oldcl);
+    x_counts = x_counts(arma::find(arma::linspace(0,K-1,K)!=oldcl),arma::find(arma::linspace(0,K-1,K)!=oldcl));
     --K;
-  }else{
-    counts=new_counts;
-    x_counts=new_ec;
   }
-
+  
+  
 }
 
 
@@ -133,29 +133,15 @@ double Sbm::delta_merge(int k, int l){
   new_ec.row(l) = new_ec.row(k)+new_ec.row(l);
   List old_stats = List::create(Named("counts", counts), Named("x_counts", x_counts));
   List new_stats = List::create(Named("counts", new_counts), Named("x_counts", new_ec));
-  double delta = icl(new_stats,k,l)-icl(old_stats,k,l);
-  
-  
+  double delta = icl_emiss(new_stats,k,l,true)-icl_emiss(old_stats,k,l,false);
   return delta;
+  
 }
 
 
-void Sbm::merge_update(int k,int l){
-  cl(arma::find(cl==k))=arma::ones(counts(k),1)*l;
-  cl.elem(arma::find(cl>k))=cl.elem(arma::find(cl>k))-1;
-  counts(l) = counts(k)+counts(l);
-  counts    = counts.elem(arma::find(arma::linspace(0,K-1,K)!=k));
-
-  x_counts.col(l) = x_counts.col(k)+x_counts.col(l);
-  x_counts.row(l) = x_counts.row(k)+x_counts.row(l);
-  x_counts = x_counts(arma::find(arma::linspace(0,K-1,K)!=k),arma::find(arma::linspace(0,K-1,K)!=k));
-  --K;
-}
-
 double Sbm::delta_merge_correction(int k,int l,int obk,int obl,const List & old_stats){
   // here old refers to the stats before the fusion between obk and obl
-  //Rcout << "Je calculs des corrections !!" << std::endl;
-  //Rcout << obk << "---- " << obl << std::endl;
+
   int a,b,ao,bo,lo;
   double icl_cor = 0;
   int cc, cc_old;
@@ -174,7 +160,7 @@ double Sbm::delta_merge_correction(int k,int l,int obk,int obl,const List & old_
   }
   for(int i=0;i<2;i++){
     for (int j=0;j<2;j++){
-
+      
       
       a = kl(i);
       b = mkl(j);
@@ -182,7 +168,7 @@ double Sbm::delta_merge_correction(int k,int l,int obk,int obl,const List & old_
         b=b-1;
       }
       
-
+      
       
       // new stats no fusion k/l
       if(j==1){
@@ -206,7 +192,7 @@ double Sbm::delta_merge_correction(int k,int l,int obk,int obl,const List & old_
       if(ao>=obk){
         ao=ao+1;
       }
-
+      
       // old stats no fusion k/l
       cc_old = old_counts(ao)*old_counts(bo);
       icl_cor += lgamma(a0+old_x_counts(ao,bo))+lgamma(b0+cc_old-old_x_counts(ao,bo))+lgamma(a0+b0)-lgamma(a0)-lgamma(b0)-lgamma(a0+b0+cc_old);
@@ -223,10 +209,20 @@ double Sbm::delta_merge_correction(int k,int l,int obk,int obl,const List & old_
       
     }
   }
-
   return icl_cor;
+}
+
+
+
+void Sbm::merge_update(int k,int l){
+  counts(l) = counts(k)+counts(l);
+  counts.shed_row(k);
   
+  x_counts.col(l) = x_counts.col(k)+x_counts.col(l);
+  x_counts.row(l) = x_counts.row(k)+x_counts.row(l);
+  x_counts = x_counts(arma::find(arma::linspace(0,K-1,K)!=k),arma::find(arma::linspace(0,K-1,K)!=k));
   
+  --K;
 }
 
 
